/*
    Copyright (C) 2014 RedoX <dev@redox.ws>
    This file is part of 4Sense.

        4Sense is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        4Sense is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with 4Sense.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _4Sense_H
#define _4Sense_H

#define nInputs 4															  /* Nb of Touch pads */
#define TFuzz 	0x90														   /* Touch fuzziness */

#define tLeft 	0x1
#define tUp 	0x2
#define tRight 	0x4
#define tDown 	0x8

#define nsSeq 	8														/* Secret sequence length */
#define ngSeq 	16														  /* Game sequence length */

#define nLeds	4																	/* Nb of LEDs */
#define lDur	10											   /* LEDs on duration on touch press */

/*
	Attiny84 pinout:
		Touch pads are connected through 10kOhms resistors
		LEDs have 150Ohms current-limiting resistors 
	Vcc 	Power	<= 1uF => Gnd
	Gnd		Ground
	PA0		Anode Top left
	PA1		Touch Up
	PA2		Anode Top righ
	PA3		HiZ	(Gnd)
	PA4		Anode Bottom right
	PA5		Touch Right
	PA6		Touch Left
	PA7		Touch Bottom
	PB0		Cathode Bottom left
	PB1		Nc
	PB2		Nc
	PB3		/Reset  <= 3.9lkOhms => Vcc
*/

/* Touch stuff */
static touch_channel_t Btns[] = {
	{
		.mux = 6,
		.port = &PORTA,
		.portmask = (1<<PA6),
	},
	{
		.mux = 1,
		.port = &PORTA,
		.portmask = (1<<PA1),
	},
    {
		.mux = 5,
		.port = &PORTA,
		.portmask = (1<<PA5),
	},
	{
		.mux = 7,
		.port = &PORTA,
		.portmask = (1<<PA7),
	},
};

/* Update the LEDs. Input is a bitmask: ex: 0b0010 */
void setLeds(uint8_t leds);
#endif
