/*
    Copyright (C) 2014 RedoX <dev@redox.ws>
    This file is part of 4Sense.

        4Sense is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        4Sense is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with 4Sense.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "touch.h"
#include "main.h"

static const uint8_t sSeq[nsSeq] = {tLeft,tRight,tLeft,tRight,tUp,tUp,tDown,tDown};  /* Konami =) */

int main(void) {
	uint16_t sRefs[nInputs] = { 0 };
	uint8_t Input,rInput,Leds,i,ticks,sSeqIdx,gSeq[ngSeq],gSeqIdx,gRun,gDif;
		/* Init CPU *******************************************************************************/
	wdt_enable(WDTO_8S);
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
		/* Init LEDs ******************************************************************************/
	PORTA = 0; DDRA = 0b00010101;								   /* Port A outputs, inputs, Low */
	PORTB = 0b01; DDRB = 0x001;												/* Port B output, Low */
	
	for(i=0;i<2*nLeds;i++) {
		setLeds(1<<(i%nLeds));_delay_ms(200); wdt_reset();							 /* LED cycle */
	}
		/* Init Touch *****************************************************************************/
    touch_init();
	for(i=0;i<nInputs;i++) {
		wdt_reset();
		sRefs[i] = touch_measure(&Btns[i])+TFuzz;				   /* Store reference + Fuzziness */
	}
	setLeds(0b1111);_delay_ms(250);setLeds(0);							 /* All LEDs on, then off */
		/* Init Rand, variables *******************************************************************/
	srand(sRefs[0]%sRefs[1]);														 /* Rand Init */
	
	sSeqIdx = 0;														 /* Secret sequence index */
	Input = 0;																	   /* Inputs bits */
	Leds = 0; ticks = 0;													 /* Leds values, time */
	gDif = 3; gSeqIdx = 0; gRun = 0; 												 /* Game init */
	while (1) {
		wdt_reset();
		rInput = 0;															   /* Temporary input */
		PRR &= ~_BV(PRADC);											  /* Disable ADC power-saving */
		for(i=0;i<nInputs;i++) {
			setLeds(0);
			if(touch_measure(&Btns[i]) > sRefs[i])							 /* Touch pad pressed */
				rInput |= _BV(i);
			setLeds(Leds); _delay_ms(10);					   /* Turn on Leds (avoid flickering) */
		}
		PRR |= _BV(PRADC);												/* Enable ADC powersaving */
		if(rInput && Input != rInput) {								/* Input and different => New */
			Leds ^= rInput;													   /* Update the LEDs */
			ticks = (Leds) ? lDur : 0;									   /* Update LEDs timeout */
		}
		Input = rInput;																 /* Old = New */
		if(ticks)
			ticks--;														 /* Keeps the LEDs on */
		else {
			if(Leds) {																 /* LEDs on ? */
				setLeds(0); _delay_ms(50);									  /* Turn off the LED */
				if(Leds == sSeq[sSeqIdx])	sSeqIdx++;	else	sSeqIdx = 0;   /* Secret sequence */
				if(gRun) {														/* Game running ? */
					if(Leds == gSeq[gSeqIdx])								 /* Combination, okay */
						gSeqIdx++;	
					else {																  /* Lost */
						gSeqIdx = 0; 													 /* Reset */
						gRun=0;													   /* Cancel game */
						gDif=3;												  /* Reset difficulty */
						for(i=Leds;i<Leds+nLeds;i++) {
							setLeds(1<<(i%nLeds)); _delay_ms(150);			/* Backward LED cycle */
						}
					}
					if(gRun && gSeqIdx >= gDif) {									/* Game won ! */
						setLeds(0b1111); _delay_ms(250); setLeds(0); wdt_reset(); _delay_ms(250);
						setLeds(0b1111); _delay_ms(250); setLeds(0); wdt_reset();
						gSeqIdx = 0; 													 /* Reset */
						gRun=0;													   /* Cancel game */
						if(gDif<ngSeq) gDif++;							   /* Increase difficulty */
					}
					
				} else if(Leds == (tUp|tDown|tLeft|tRight)) {				  /* Start a new game */
					for(i=0;i<gDif;i++) {
						setLeds(0b10); _delay_ms(150); 
						setLeds(0); wdt_reset(); _delay_ms(150);			  /* Blink difficulty */
					}
					_delay_ms(500);
					for(i=0;i<gDif;i++) {
						gSeq[i] = (1<<(rand()%4));						   /* Random new sequence */
						setLeds(gSeq[i]); _delay_ms(500); 
						setLeds(0); wdt_reset(); _delay_ms(500);				 /* Display it ^^ */
					}
					gRun=1;
				}
				if(sSeqIdx >= nsSeq) {								/* Secret sequence complete ! */
																				/* Blink all LEDs */
					setLeds(0b1111); _delay_ms(250); setLeds(0); wdt_reset(); _delay_ms(250);
					setLeds(0b1111); _delay_ms(250); setLeds(0); wdt_reset(); _delay_ms(250);
					for(i=0;i<3*nLeds+1;i++) {
						setLeds(1<<(i%nLeds));_delay_ms(100);				 /* Forward LED cycle */
						wdt_reset();
					}
																				/* Blink all LEDs */
					setLeds(0b1111); _delay_ms(250); setLeds(0); wdt_reset(); _delay_ms(250);
					for(i=3*nLeds-1;i<3*nLeds;i--) {
						setLeds(1<<(i%nLeds));_delay_ms(100);				/* Backward LED cycle */
						wdt_reset();
					}
																				/* Blink all LEDs */
					setLeds(0b1111); _delay_ms(250); setLeds(0); wdt_reset(); _delay_ms(250);
					setLeds(0b1111); _delay_ms(250); setLeds(0); wdt_reset(); 
				}
				Leds = 0;																 /* Clear */
			}
			/* Go to Sleep ! */
			ADCSRA &= ~_BV(ADEN);  												   /* Disable ADC */
			GIMSK |= _BV(PCIE0); PCMSK0 = 0b11100010;					/* Enable PCINT interrupt */
			MCUSR = 0; WDTCSR |= _BV(WDCE)|_BV(WDE); WDTCSR = _BV(WDIE)| WDTO_8S;		   /* WDT */
			sleep_enable();
			sei();
			sleep_cpu();													   /* Go to sleep !!! */
			cli();
			sleep_disable();
			wdt_enable(WDTO_8S);											/* Restore normal WDT */
			GIMSK &= ~_BV(PCIE0);									   /* Disable PCINT interrupt */
			ADCSRA |= _BV(ADEN);													/* Enable ADC */
			
		}
	}
}

/* Update the LEDs. Input is a bitmask: ex: 0b0010 */
void setLeds(uint8_t leds) {
	if(leds & 0b0001) PORTA |= 0b1; else PORTA &= ~0b1;								  /* Top Left */
	if(leds & 0b0010) PORTA |= 0b100; else PORTA &= ~0b100;							 /* Top Right */
	if(leds & 0b0100) PORTA |= 0b10000; else PORTA &= ~0b10000;					  /* Bottom Right */
	if(leds & 0b1000) PORTB &= ~0b1; else PORTB |= 0b1;							   /* Bottom Left */
}

/* Dummy ISR handlers */
ISR(WDT_vect) {
}

ISR(PCINT0_vect) {	
}
