/*
    Copyright (C) 2014 RedoX <dev@redox.ws>
    This file is part of 4Sense.

        4Sense is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        4Sense is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with 4Sense.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/wdt.h>

#include "touch.h"

void touch_init(void){
    ADMUX &=  ~(_BV(REFS0)|_BV(REFS1)); 										/* Reference: VCC */
    ADCSRA |= _BV(ADEN);															/* Enable ADC */
}
	
void adc_channel(uint8_t channel) {
    ADMUX &= ~(0b111111);															 /* Clear Mux */
    ADMUX |= 0b111111 & channel;													   /* Set new */
}

uint16_t adc_get(void) {
    ADCSRA |= (1<<ADSC); 													  /* Start conversion */
    while(!(ADCSRA & (1<<ADIF)));								 /* Wait for conversion to finish */
    ADCSRA |= (1<<ADIF); 														/* Clear the flag */
    return ADC;
}

uint16_t touch_measure(touch_channel_t *channel) {
    uint8_t i;
    uint16_t retval=0;
    for (i=0;i<4;i++){						 /* Average 4 measurements, just to smooth things out */
		*(channel->port) |= channel->portmask; 									 /* Enable pullup */
		_delay_us(600);							 /* Wait a little bit so the internal cap charges */
		*(channel->port) &= ~(channel->portmask); 								/* Disable pullup */
		adc_channel(0b100000); 										 /* Set ADC channel to Ground */
		adc_get(); 					   /* Do a measurement to discharge the sampling/internal cap */
		adc_channel(channel->mux); 								 /* Set mux to the actual channel */
		retval += adc_get(); 													/* Read the value */
    }
    return retval/4;													   /* Returns the average */
}